@@ -6,13 +6,12 @@ simulations        <- 100
 
 bandit             <- BasicBernoulliBandit$new(prob_per_arm)
 
-agents             <- list(Agent$new(RandomPolicy$new(), bandit),
-                           Agent$new(OraclePolicy$new(), bandit),
-                           Agent$new(EpsilonGreedyPolicy$new(0.1), bandit),
-                           Agent$new(ThompsonSamplingPolicy$new(1.0, 1.0), bandit),
+agents             <- list(Agent$new(EpsilonGreedyPolicy$new(0.1), bandit),
+                           Agent$new(ThompsonSamplingPolicy$new(1, 1), bandit),
                            Agent$new(Exp3Policy$new(0.1), bandit),
                            Agent$new(GittinsBrezziLaiPolicy$new(), bandit),
-                           Agent$new(UCB1Policy$new(), bandit))
+                           Agent$new(UCB1Policy$new(), bandit),
+                           Agent$new(UCB2Policy$new(0.1), bandit))
 
 simulation         <- Simulator$new(agents, horizon, simulations)
 history            <- simulation$run()
@@ -20,4 +19,3 @@ history            <- simulation$run()
 plot(history, type = "cumulative")
 
 summary(history)
-
